/********************************************************************
 *  Licensed Materials - Property of IBM
 *
 *  (c) Copyright IBM Corp. 2009 All Rights Reserved
 *
 *  US Government Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with
 *  IBM Corp.
 ********************************************************************/

package examples.authentication;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import com.ibm.itim.authentication.AuthenticationFailedException;
import com.ibm.itim.authentication.AuthenticationProvider;
import com.ibm.itim.authentication.simple.SimpleAuthenticationProvider;
import com.ibm.itim.authentication.Authenticator;
import com.ibm.itim.authentication.ConfigurationException;
import com.ibm.itim.authentication.Credentials;
import com.ibm.itim.dataservices.model.system.SystemUser;
import com.ibm.itim.logging.SystemLog;

/**
 * Provider assumes that authentication is provided elsewhere, but looks up the
 * enRole system user and checks that an account exists.
 */
public class ADAuthenticationProvider implements AuthenticationProvider {

    private static String AD_AUTH_PROPERTIES = "enRoleADAuthentication.properties";
    private static String PROP_DEFAULT_AD_DOMAIN = "defaultActiveDirectoryDomain";
    private static String PROP_USE_FULLY_QUALIFIED_USER_NAME = "useFullyQualifiedUserName";
    private static String PROP_DEFAULT_DNS_SERVER = "defaultDnsServer";

    private Properties props = null;
    private Boolean useFullyQualifiedName;
    private String defaultActiveDirectoryDomain;
    private String defaultDnsServer;

    /** Creates new ADAuthenticationProvider */
    public ADAuthenticationProvider(Hashtable env) {
	    loadProperties();
    }

    /**
     * Authenticates a user with the given credentials. The success or failure
     * of the authentication and the reason for failure (if any) is returned to
     * the caller.
     * @param userCredentials The user's credentials to be authenticated.
     * @return SystemUser if authenticated successfully
     * @exception AuthenticationFailedException thrown if the authentication attempt failed to invalid
     *            user credentials.
     * @exception ConfigurationException thrown if the required credentials are not provided or the
     *            provided credentials are malformed.
     */
    public SystemUser authenticate(Credentials userCredentials)
            throws AuthenticationFailedException, ConfigurationException {
        String uid = (String) userCredentials
                .getCredential(Authenticator.PRINCIPAL);
        String password = (String) userCredentials
                .getCredential(Authenticator.KEY);
        String tenantDN = (String) userCredentials
                .getCredential(Authenticator.TENANT);

        String domain = null;
        if (useFullyQualifiedName.booleanValue()) {
            if (uid.indexOf("@") != -1 ) {
                domain = (uid.split("@"))[1];
            } else {
                domain = defaultActiveDirectoryDomain;
                uid = uid + "@" + domain;
            }
        } else {
            domain = defaultActiveDirectoryDomain;
            uid = uid + "@" + domain;
        }
        SystemLog.getInstance().logInformation(this,
            "Active Directory Domain is " + domain);

        // Determine if the user is an ITIM user
        ADAuthenticationHelper helper = new ADAuthenticationHelper(defaultDnsServer);
        SystemUser user = helper.findUser(uid, tenantDN);
        userCredentials.setCredential(Authenticator.PRINCIPAL,uid);

        // Check if password is blank then throw exception
        if(password.equals(""))
            throw new AuthenticationFailedException("Either no such user '"
                    + uid + "' or incorrect password");

        // Locate the LDAP Server
        helper.checkDomain(domain);

        // Authenticate the user against Active Directory
        helper.adAuthenticate(uid, password, domain);
        return user;
    }

    /**
     * Find the path of 'data' directory present in the installed ITIM
     * @return absolute path of the data directory
     */
    private String findDataDir() {
        String classPath = System.getProperty("java.class.path");
        StringTokenizer st = new StringTokenizer(classPath, File.pathSeparator);
        while (st.hasMoreElements()) {
            String dirName = st.nextToken();
            File file = new File(dirName, "Properties.properties");
            if (file.isFile())
                return dirName;
        }
        return null;
    }

    /**
     * load all the properties from AD_AUTH_PROPERTIES.
     */
    private void loadProperties(){
        try {
            String dataDir = findDataDir();
            URL propsURL = new URL("file:" + dataDir + "/" + AD_AUTH_PROPERTIES);
            props = new Properties();
            props.load(propsURL.openStream());
            useFullyQualifiedName = Boolean.valueOf(props.getProperty(
                    PROP_USE_FULLY_QUALIFIED_USER_NAME, "false"));
            defaultActiveDirectoryDomain = props.getProperty(
                    PROP_DEFAULT_AD_DOMAIN, null);
            defaultDnsServer = props.getProperty(
                    PROP_DEFAULT_DNS_SERVER, null);
         }catch (IOException e) {
            SystemLog.getInstance().logError(this,"Failed to load Active Directory authentication properties");
        }
    }

}
